@@ -1,6 +1,6 @@
 /*
  Copyright (C) Intel Corp.  2006.  All Rights Reserved.
- Intel funded Tungsten Graphics (http://www.tungstengraphics.com) to
+ Intel funded Tungsten Graphics to
  develop this 3D driver.
 
  Permission is hereby granted, free of charge, to any person obtaining
@@ -26,7 +26,7 @@
  **********************************************************************/
  /*
   * Authors:
-  *   Keith Whitwell <keith@tungstengraphics.com>
+  *   Keith Whitwell <keithw@vmware.com>
   */
 
 #include "main/mtypes.h"
@@ -34,7 +34,7 @@
 
 #include "brw_context.h"
 #include "brw_state.h"
-
+#include "intel_buffer_objects.h"
 
 void
 brw_upload_vec4_pull_constants(struct brw_context *brw,
@@ -51,10 +51,8 @@ brw_upload_vec4_pull_constants(struct brw_context *brw,
     */
    _mesa_load_state_parameters(&brw->ctx, prog->Parameters);
 
-   if (!prog_data->nr_pull_params) {
-      if (stage_state->const_bo) {
-	 drm_intel_bo_unreference(stage_state->const_bo);
-	 stage_state->const_bo = NULL;
+   if (!prog_data->base.nr_pull_params) {
+      if (stage_state->surf_offset[surf_index]) {
 	 stage_state->surf_offset[surf_index] = 0;
 	 brw->state.dirty.brw |= brw_new_constbuf;
       }
@@ -62,32 +60,28 @@ brw_upload_vec4_pull_constants(struct brw_context *brw,
    }
 
    /* _NEW_PROGRAM_CONSTANTS */
-   drm_intel_bo_unreference(stage_state->const_bo);
-   uint32_t size = prog_data->nr_pull_params * 4;
-   stage_state->const_bo = drm_intel_bo_alloc(brw->bufmgr, "vec4_const_buffer",
-                                           size, 64);
+   uint32_t size = prog_data->base.nr_pull_params * 4;
+   drm_intel_bo *const_bo = NULL;
+   uint32_t const_offset;
+   float *constants = intel_upload_space(brw, size, 64,
+                                         &const_bo, &const_offset);
 
-   drm_intel_gem_bo_map_gtt(stage_state->const_bo);
-
-   for (i = 0; i < prog_data->nr_pull_params; i++) {
-      memcpy(stage_state->const_bo->virtual + i * 4,
-	     prog_data->pull_param[i],
-	     4);
+   for (i = 0; i < prog_data->base.nr_pull_params; i++) {
+      constants[i] = *prog_data->base.pull_param[i];
    }
 
    if (0) {
-      for (i = 0; i < ALIGN(prog_data->nr_pull_params, 4) / 4; i++) {
-	 float *row = (float *)stage_state->const_bo->virtual + i * 4;
-	 printf("const surface %3d: %4.3f %4.3f %4.3f %4.3f\n",
-		i, row[0], row[1], row[2], row[3]);
+      for (i = 0; i < ALIGN(prog_data->base.nr_pull_params, 4) / 4; i++) {
+	 float *row = &constants[i * 4];
+	 fprintf(stderr, "const surface %3d: %4.3f %4.3f %4.3f %4.3f\n",
+                 i, row[0], row[1], row[2], row[3]);
       }
    }
 
-   drm_intel_gem_bo_unmap_gtt(stage_state->const_bo);
-
-   brw_create_constant_surface(brw, stage_state->const_bo, 0, size,
+   brw_create_constant_surface(brw, const_bo, const_offset, size,
                                &stage_state->surf_offset[surf_index],
                                false);
+   drm_intel_bo_unreference(const_bo);
 
    brw->state.dirty.brw |= brw_new_constbuf;
 }
@@ -130,7 +124,8 @@ brw_upload_vs_ubo_surfaces(struct brw_context *brw)
 {
    struct gl_context *ctx = &brw->ctx;
    /* _NEW_PROGRAM */
-   struct gl_shader_program *prog = ctx->Shader.CurrentVertexProgram;
+   struct gl_shader_program *prog =
+      ctx->_Shader->CurrentProgram[MESA_SHADER_VERTEX];
 
    if (!prog)
       return;
@@ -154,7 +149,8 @@ brw_upload_vs_abo_surfaces(struct brw_context *brw)
 {
    struct gl_context *ctx = &brw->ctx;
    /* _NEW_PROGRAM */
-   struct gl_shader_program *prog = ctx->Shader.CurrentVertexProgram;
+   struct gl_shader_program *prog =
+      ctx->_Shader->CurrentProgram[MESA_SHADER_VERTEX];
 
    if (prog) {
       /* CACHE_NEW_VS_PROG */
